# Copyright:	Public domain.
# Filename:	ASSEMBLY_AND_OPERATION_INFORMATION.s
# Purpose:	Part of the source code for Artemis (i.e., Colossus 3),
#		build 072.  It is part of the source code for the
#		Command Module's (CM) Apollo Guidance Computer (AGC),
#		possibly for Apollo 15.
# Assembler:	yaYUL
# Contact:	Ron Burkey <info@sandroid.org>.
# Website:	www.ibiblio.org/apollo/index.html
# Mod history:	12/21/04 RSB.	Created.
#		05/14/05 RSB	Corrected website reference above.
#
# The contents of the "Artemis072" files, in general, are transcribed
# from scanned page images contributed by D. Thrust.  Notations on this
# document read, in part:
#
#	Assemble revision 249 of AGC program Colossus by NASA
#	2021111-041.  October 28, 1968.  
#
#	THIS AGC PROGRAM SHALL ALSO BE REFERRED TO AS
#				COLOSSUS 3
#	THIS PROGRAM IS INTENDED FOR USE IN THE CM AS SPECIFIED
#	IN REPORT R-577.  THIS PROGRAM WAS PREPARED UNDER OSR
#	PROJECT 55-23890, SPONSORED BY THE MANNED SPACECRAFT
#	CENTER OF THE NATIONAL AERONAUTICS AND SPACE
#	ADMINISTRATION THROUGH CONTRACT NAS 9-4065 WITH THE
#	CHARLES STARK DRAPER LABORATORY, MASSACHUSETTS INSTITUTE OF
#	TECHNOLOGY, CAMBRIDGE, MASS.
#
# Please report any errors to info@sandroid.org.
#
# In some cases, where the source code for Luminary 131 or for
# Colossus 249 overlaps that of Artemis 072, this code is instead copied
# from the corresponding Luminary 131 or Colossus 249
# source file, and then is proofed to incorporate any changes.

# Page 1

#########################################################################
#									#
#	THIS AGC PROGRAM SHALL ALSO BE REFERRED TO AS:			#
#									#
#				COLOSSUS 3				#
#									#
#	THIS PROGRAM IS INTENDED FOR USE IN THE CM AS SPECIFIED		#
#	IN REPORT R-577.  THIS PROGRAM WAS PREPARED UNDER OSR		#
#	PROJECT 55-23890, SPONSORED BY THE MANNED SPACECRAFT		#
#	CENTER OF THE NATIONAL AERONAUTICS AND SPACE			#
#	ADMINISTRATION THROUGH CONTRACT NAS 9-4065 WITH THE		#
#	CHARLES STARK DRAPER LABORATORY, MASSACHUSETTS INSTITUTE OF	#
#	TECHNOLOGY, CAMBRIDGE, MASS.					#
#									#
#########################################################################

#	SUBMITTED:
#		A. G. ENGEL JR, COLOSSUS PROGRAMMING LEADER
#		APOLLO GUIDANCE AND NAVIGATION

#	APPROVED:
#		M. H. HAMILTON, DIRECTOR, MISSION PROGRAM DEVELOPMENT
#		APOLLO GUIDANCE AND NAVIGATION PROGRAM

#	APPROVED:
#		S. L. COPPS, COLOSSUS PROJECT MANAGER
#		APOLLO GUIDANCE AND NAVIGATION PROGRAM

#	APPROVED:
#		R. H. BATTIN, DIRECTOR, MISSION DEVELOPMENT
#		APOLLO GUIDANCE AND NAVIGATION PROGRAM

#	APPROVED:
#		D. G. HOAG, DIRECTOR
#		APOLLO GUIDANCE AND NAVIGATION PROGRAM

#	APPROVED:
#		R. R. RAGAN, DEPUTY DIRECTOR
#		INSTRUMENTATION LABORATORY

# Page 2?

# ASSEMBLY AND OPERATIONS INFORMATION
# TAGS FOR RELATIVE SETLOC AND BLANK BANK CARDS
# ABSOLUTE LOCATIONS FOR UPDATES
# SUBROUTINE CALLS

#	ERASTOTL
#		ERASABLE ASSIGNMENTS
#		CHECK EQUALS LIST
#	DIOGENES
#		INTERRUPT LEAD INS
#		T4RUPT PROGRAM
#		DOWNLINK LISTS
#		FRESH START AND RESTART
#		RESTART TABLES
#		SXTMARK
#		EXTENDED VERBS
#		PINBALL NOUN TABLES
#		CSM GEOMETRY
#		IMU COMPENSATION PACKAGE
#		PINBALL GAME BUTTONS AND LIGHTS
#		R60,R62
#		ANGLFIND
#		GIMBAL LOCK AVOIDANCE
#		KALCMANU STEERING
#		SYSTEM TEST STANDARD LEAD INS
#		IMU CALIBRATION AND ALIGNMENT
#	MEDUSA
#		GROUND TRACKING DETERMINATION PROGRAM - P21
#		P34-P34, P74-P75
#		R31
#		P76
#		R30
#	MENELAUS
#		P15
#		P11
#		P20-P25
#		P30,P31
#		P32-P33, P72-P73
#		P40-P47
#		P51-P53
#		LUNAR AND SOLAR EPHEMERIDES SUBROUTINES
#		P61-P67
#		SERVICER207
#		ENTRY LEXICON
#		REENTRY CONTROL
#		CM BODY ATTITUDE
#		P37, P70
#		S-BAND ANTENNA FOR CM
#	ULYSSES
# Page 3?
#		TVCINITIALIZE
#		TVCEXECUTIVE
#		TVCMASSPROP
#		TVCRESTARTS
#		TVCDAPS
#		TVCROLLDAP
#		MYSUBS
#		RCS-CSM DIGITAL AUTOPILOT
#		AUTOMATIC MANEUVERS
#		RCS-CSM DAP EXECUTIVE PROGRAMS
#		JET SELECTION LOGIC
#		CM ENTRY DIGITAL AUTOPILOT
#	ZEUS
#		DOWN-TELEMETRY PROGRAM
#		INTER-BANK COMMUNICATION
#		INTERPRETER
#		FIXED-FIXED CONSTANT POOL
#		INTERPRETIVE CONSTANTS
#		SINGLE PRECISION SUBROUTINES
#		EXECUTIVE
#		WAITLIST
#		LATITUDE LONGITUDE SUBROUTINES
#		PLANETARY INERTIAL ORIENTATION
#		MEASUREMENT INCORPORATION
#		CONIC SUBROUTINES
#		INTEGRATION INITIALIZATION
#		ORBITAL INTEGRATION
#		INFLIGHT ALIGNMENT ROUTINES
#		POWERED FLIGHT SUBROUTINES
#		TIME OF FREE FALL
#		STAR TABLES
#		AGC BLOCK TWO SELF-CHECK
#		PHASE TABLE MAINTENANCE
#		RESTARTS ROUTINE
#		IMU MODE SWITCHING ROUTINES
#		KEYRUPT, UPRUPT
#		DISPLAY INTERFACE ROUTINES
#		SERVICE ROUTINES
#		ALARM AND ABORT
#		UPDATE PROGRAM
#		RT8 OP CODES

# SYMBOL TABLE LISTING
# UNREFERENCED SYMBOL LISTING
# ERASABLE & EQUALS CROSS-REFERENCE TABLE
# SUMMARY OF SYMBOL TABLE LISTINGS
# MEMORY TYPE & AVAILABILITY DISPLAY
# COUNT TABLE
# PARAGRAPHS GENERATED FOR THIS ASSEMBLY
# Page 4?
# OCTAL LISTING
# OCCUPIED LOCATIONS TABLE
# SUBROS CALLED & PROGRAM STATUS

# Page 5?
# VERB LIST FOR CSM

# REGULAR VERBS

# 00	NOT IN USE
# 01	DISPLAY OCTAL COMP 1 IN R1
# 02	DISPLAY OCTAL COMP 2 IN R1
# 03	DISPLAY OCTAL COMP 3 IN R1
# 04	DISPLAY OCTAL COMP 1,2 IN R1,R2
# 05	DISPLAY OCTAL COMP 1,2,3 IN R1,R2,R3
# 06	DISPLAY DECIMAL IN R1 OR R1,R2 OR R1,R2,R3
# 07	DISPLAY DP DECIMAL IN R1,R2 (TEST ONLY)
# 08
# 09
# 10
# 11	MONITOR OCTAL COMP 1 IN R1
# 12 	MONITOR OCTAL COMP 2 IN R1
# 13	MONITOR OCTAL COMP 3 IN R1
# 14	MONITOR OCTAL COMP 1,2, IN R1,R2
# 15	MONITOR OCTAL COMP 1,2,3 IN R1,R2,R3
# 16	MONITOR DECIMAL IN R1 OR R1,R2 OR R1,R2,R3
# 17	MONITOR DP DECIMAL IN R1,R2 (TEST ONLY)
# 18
# 19
# 20
# 21	LOAD COMPONENT 1 INTO R1
# 22	LOAD COMPONENT 2 INTO R1
# 23	LOAD COMPONENT 3 INTO R1
# 24	LOAD COMPONENT 1,2 INTO R1,R2
# 25	LOAD COMPONENT 1,2,3 INTO R1,R2,R3
# 26
# 27	DISPLAY FIXED MEMORY
# 28
# 29
# 30	REQUEST EXECUTIVE
# 31	REQUEST WAITLIST
# 32	RECYCLE PROGRAM
# 33	PROCEED WITHOUT DSKY INPUTS
# 34	TERMINATE FUNCTION
# 35	TEST LIGHTS
# 36	REQUEST FRESH START
# 37	CHANGE PROGRAM (MAJOR MODE)
# 38
# 39

# Page 37
# SUBROUTINE CALLS

		SUBRO	ERASTOTL
		SUBRO	DIOGENES
		SUBRO	MEDUSA
		SUBRO	MENELAUS
		SUBRO	ULYSSES
		SUBRO	ZEUS



